from __future__ import annotations

from polars import DataFrame
from polars.type_aliases import PolarsDataType, SchemaDict

from utilities.typing import SequenceStrs


def check_dataframe(
    df: DataFrame,
    /,
    *,
    columns: SequenceStrs | None = None,
    dtypes: list[PolarsDataType] | None = None,
    height: int | None = None,
    schema: SchemaDict | None = None,
    shape: tuple[int, int] | None = None,
    width: int | None = None,
) -> None:
    if (columns is not None) and (df.columns != list(columns)):
        msg = f"{df=}, {columns=}"
        raise DataFrameColumnsError(msg)
    if (dtypes is not None) and (df.dtypes != dtypes):
        msg = f"{df=}, {dtypes=}"
        raise DataFrameDTypesError(msg)
    if (height is not None) and (df.height != height):
        msg = f"{df=}"
        raise DataFrameHeightError(msg)
    if (schema is not None) and (df.schema != schema):
        msg = f"{df=}"
        raise DataFrameSchemaError(msg)
    if (shape is not None) and (df.shape != shape):
        msg = f"{df=}"
        raise DataFrameShapeError(msg)
    if (width is not None) and (df.width != width):
        msg = f"{df=}"
        raise DataFrameWidthError(msg)


class DataFrameColumnsError(ValueError):
    """Raised when a DataFrame has the incorrect columns."""


class DataFrameDTypesError(ValueError):
    """Raised when a DataFrame has the incorrect dtypes."""


class DataFrameHeightError(ValueError):
    """Raised when a DataFrame has the incorrect height."""


class DataFrameSchemaError(ValueError):
    """Raised when a DataFrame has the incorrect schema."""


class DataFrameShapeError(ValueError):
    """Raised when a DataFrame has the incorrect shape."""


class DataFrameWidthError(ValueError):
    """Raised when a DataFrame has the incorrect width."""



from __future__ import annotations

from locale import LC_CTYPE, LC_TIME, setlocale

from pytest import mark, param

from utilities.locale import atof, get_locale_for_platform, override_locale
from utilities.platform import SYSTEM, System


class TestAToF:
    @mark.parametrize(
        ("text", "expected"),
        [
            param("0.00", 0.0),
            param("1.23", 1.23),
            param("12.34", 12.34),
            param("123.45", 123.45),
            param("1,234.56", 1234.56),
        ],
    )
    def test_main(self, *, text: str, expected: float) -> None:
        plat_locale = get_locale_for_platform("en_US")
        assert atof(text, locale=plat_locale) == expected


class TestGetLocaleForPlatform:
    def test_main(self) -> None:
        plat_locale = get_locale_for_platform("en_US")
        _ = setlocale(LC_CTYPE, locale=plat_locale)


class TestOverrideLocale:
    @mark.skipif(condition=SYSTEM is System.windows, reason="non-Windows only")
    def test_main(self) -> None:
        plat_locale = get_locale_for_platform("en_US")
        with override_locale(locale=plat_locale):
            assert LC_TIME in {2, 5}



from __future__ import annotations

from pathlib import Path
from re import search
from subprocess import CalledProcessError, check_call

from pytest import mark, raises

from utilities.platform import SYSTEM, System
from utilities.subprocess import (
    MultipleActivateError,
    NoActivateError,
    _address_already_in_use_pattern,
    get_shell_output,
    tabulate_called_process_error,
)
from utilities.text import strip_and_dedent


class TestGetShellOutput:
    @mark.skipif(condition=SYSTEM is System.windows, reason="non-Windows only")
    def test_main(self) -> None:
        output = get_shell_output("ls")
        assert any(line == "pyproject.toml" for line in output.splitlines())

    @mark.skipif(condition=SYSTEM is System.windows, reason="non-Windows only")
    def test_activate(self, *, tmp_path: Path) -> None:
        venv = tmp_path.joinpath(".venv")
        activate = venv.joinpath("activate")
        activate.parent.mkdir(parents=True)
        activate.touch()
        _ = get_shell_output("ls", cwd=venv, activate=venv)

    def test_no_activate(self, *, tmp_path: Path) -> None:
        venv = tmp_path.joinpath(".venv")
        with raises(NoActivateError):
            _ = get_shell_output("ls", cwd=venv, activate=venv)

    def test_multiple_activates(self, *, tmp_path: Path) -> None:
        venv = tmp_path.joinpath(".venv")
        for i in range(2):
            activate = venv.joinpath(str(i), "activate")
            activate.parent.mkdir(parents=True)
            activate.touch()
        with raises(MultipleActivateError):
            _ = get_shell_output("ls", cwd=venv, activate=venv)


class TestAddressAlreadyInUsePattern:
    def test_pattern(self) -> None:
        pattern = _address_already_in_use_pattern()
        text = "OSError: [Errno 98] Address already in use"
        assert search(pattern, text) is not None


class TestTabulateCalledProcessError:
    @mark.skipif(condition=SYSTEM is System.windows, reason="non-Windows only")
    def test_main(self) -> None:
        def which() -> None:
            _ = check_call(["which"], text=True)  # noqa: S603, S607

        try:
            which()
        except CalledProcessError as error:
            result = tabulate_called_process_error(error)
            expected = """
                cmd        ['which']
                returncode 1
                stdout     None
                stderr     None
            """
            assert result == strip_and_dedent(expected)
        else:
            with raises(CalledProcessError):
                which()



from __future__ import annotations

from polars import DataFrame, Float64, Int64
from pytest import raises

from utilities.polars import (
    DataFrameColumnsError,
    DataFrameDTypesError,
    DataFrameHeightError,
    DataFrameSchemaError,
    DataFrameShapeError,
    DataFrameWidthError,
    check_dataframe,
)


class TestCheckDataFrame:
    def test_main(self) -> None:
        df = DataFrame()
        check_dataframe(df)

    def test_columns_pass(self) -> None:
        df = DataFrame({"value": [0.0]})
        check_dataframe(df, columns=["value"])

    def test_columns_error(self) -> None:
        df = DataFrame({"value": [0.0]})
        with raises(DataFrameColumnsError):
            check_dataframe(df, columns=["other"])

    def test_dtypes_pass(self) -> None:
        df = DataFrame({"value": [0.0]})
        check_dataframe(df, dtypes=[Float64])

    def test_dtypes_error(self) -> None:
        df = DataFrame({"value": [0.0]})
        with raises(DataFrameDTypesError):
            check_dataframe(df, dtypes=[Int64])

    def test_height_pass(self) -> None:
        df = DataFrame()
        check_dataframe(df, height=0)

    def test_height_error(self) -> None:
        df = DataFrame()
        with raises(DataFrameHeightError):
            check_dataframe(df, height=1)

    def test_schema_pass(self) -> None:
        df = DataFrame()
        check_dataframe(df, schema={})

    def test_schema_error(self) -> None:
        df = DataFrame()
        with raises(DataFrameSchemaError):
            check_dataframe(df, schema={"other": Float64})

    def test_shape_pass(self) -> None:
        df = DataFrame()
        check_dataframe(df, shape=(0, 0))

    def test_shape_error(self) -> None:
        df = DataFrame()
        with raises(DataFrameShapeError):
            check_dataframe(df, shape=(1, 1))

    def test_width_pass(self) -> None:
        df = DataFrame()
        check_dataframe(df, width=0)

    def test_width_error(self) -> None:
        df = DataFrame()
        with raises(DataFrameWidthError):
            check_dataframe(df, width=1)
